##############################################################################
#
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
#
# Copyright (c) Digital Creations.  All rights reserved.
#
# This license has been certified as Open Source(tm).
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
#
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
#
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
#
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
#
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
#
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
#
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
#
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
#
#
# Disclaimer
#
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
#
#
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
#
##############################################################################
__doc__='''ZopeStudio RDF Generation

RDF information about Zope objects, generated for use in ZopeStudio or other
IDEs. Methods in this module are added as methods to the Zope Application
class, thus making them available to the whole Zope application.

$Id: ZopeStudioRDF.py,v 1.4 2000/07/15 12:58:17 mj Exp $'''
__version__='$Revision: 1.4 $'[11:-2]

from DocumentTemplate.pDocumentTemplate import isSimpleType, isFunctionType
from DocumentTemplate.DT_Util import html_quote
from string import join, find
from urllib import quote_plus
import OFS.Application

# This method is made class method of the Zope root (Application) object
def zsTreeRoot(self, REQUEST):
    """ Return a RDF description of the Zope:TreeRoot object. The Mozilla tree
    widget is anchored at this RDF object. """
    self = REQUEST.PARENTS[0]

    REQUEST.RESPONSE.setHeader('content-type', 'text/xml')

    output = """<?xml version="1.0"?>
<rdf:RDF xml:lang="en"
         xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
         xmlns:dc="http://purl.org/metadata/dublin_core#"
         xmlns:zope="http://www.zope.org/Resources/Mozilla/RDF#"
         xmlns:nc="http://home.netscape.com/NC-rdf#">
    <rdf:Seq about="Zope:TreeRoot">
        <rdf:li>
          %s
        </rdf:li>
    </rdf:Seq>
</rdf:RDF>""" % itemRDFSnippet(self, REQUEST.URL1, 1, REQUEST, isSubItem = 0)

    return output

# Shove the above method in the Application class
OFS.Application.Application.zsTreeRoot = zsTreeRoot

# This method is made class method of the Zope root (Application) object
def zsContainerRDF(self, REQUEST):
    """ Return a RDF description of a container for use in ZopeStudio """
    self = REQUEST.PARENTS[0]

    REQUEST.RESPONSE.setHeader('content-type', 'text/xml')

    output = """<?xml version="1.0"?>
<rdf:RDF xml:lang="en"
         xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
         xmlns:dc="http://purl.org/metadata/dublin_core#"
         xmlns:zope="http://www.zope.org/Resources/Mozilla/RDF#"
         xmlns:nc="http://home.netscape.com/NC-rdf#">
    %s
</rdf:RDF>""" % containerRDFSnippet(self, REQUEST)

    return output

# Shove the above method in the Application class
OFS.Application.Application.zsContainerRDF = zsContainerRDF

# Create a snippet of RDF for a container.
# Take into account sub containers (based on the result of tpValues), and
# take into account tpURL.
def containerRDFSnippet(object, REQUEST):
    subitems = containerChildren(object, REQUEST)
    subitems = map(lambda item, R=REQUEST:
        '<rdf:li>%s</rdf:li>' % itemRDFSnippet(item[0], item[1], item[2], R),
        subitems)
    subitems = join(subitems, '\n')

    icon = getvalue(object, 'icon')
    if icon: icon = '%s/%s' % (REQUEST.BASE1, quote_plus(icon))

    isEmpty = 'true'
    if getvalue(object, 'tpValues'): isEmpty = 'false'

    return """<rdf:Seq about="%s">
            %s
        </rdf:Seq>""" % (REQUEST.URL1, subitems)

# Create a snippet of RDF for a random item. Container subitems will not be
# included.
def itemRDFSnippet(object, url, isTreeItem, REQUEST, isSubItem = 1):
    val = lambda att, obj=object: html_quote(getvalue(obj, att))

    icon = getvalue(object, 'icon')
    if icon: icon = '%s/%s' % (REQUEST.BASE1, quote_plus(icon))

    seeAlso = ''
    isEmpty = ''
    if isTreeItem:
        if isSubItem: tpURL = '/%s' % quote_plus(getvalue(object, 'tpURL'))
        else: tpURL = ''
        seeAlso = ' rdf:seeAlso="%s%s/zsContainerRDF"' % (
            REQUEST.URL1, tpURL)
        isEmpty = ' zope:treeBranchEmpty="true"'
        if getvalue(object, 'tpValues'):
            isEmpty = ' zope:treeBranchEmpty="false"'

    return """<rdf:Description about="%s"%s
        dc:Title="%s" dc:Identifier="%s" dc:Date="%s" dc:Type="%s"
        nc:Icon="%s" zope:showInTree="%s"%s/>""" % (
            url, seeAlso, val('title'), val('id'),
            val('bobobase_modification_time'), val('meta_type'), icon,
            isTreeItem, isEmpty)

# Return a list of (object, url, isTreeItem) tuples, of all subobjects of
# container. Subobjects is a join of the results of objectValues and tpValues.
# isTreeItem is set when the object appears in tpValues
def containerChildren(object, REQUEST):
    children = {}
    # First, objectValues
    if hasattr(object, 'objectValues'):
        for child in getvalue(object, 'objectValues'):
            id = getvalue(child, 'id')
            children[id] = (child, '%s/%s' % (REQUEST.URL1, quote_plus(id)), 0)
    # Next, tpValues. Any object that appears in both, will have tpURL override
    # id in the generated URL.
    if hasattr(object, 'tpValues'):
        for child in getvalue(object, 'tpValues'):
            id = getvalue(child, 'id')
            tpURL = '%s/%s' % (REQUEST.URL1,
                quote_plus(getvalue(child, 'tpURL')))
            children[id] = (child, tpURL, 1)

    # return the results sorted on id.
    ids = children.keys()
    ids.sort()
    return map(lambda key, chn=children: chn[key], ids)

# Return the value of an attribute, be it a standard type, callable, or a
# DocumentTemplate. Returns an empty string if the attribute is not present.
# Code based on DocumentTemplate.pDocumentTemplate.TemplateDict.__getitem__
def getvalue(object, attribute, simpleType=isSimpleType,
             functionType=isFunctionType):
    if hasattr(object, attribute):
        value = getattr(object, attribute)

        # Try and call it if not a simple. Take care of Document Templates.
        if not simpleType(type(value)):
            if hasattr(value,'isDocTemp') and value.isDocTemp:
                return value(None, object) # call Document Template
            elif functionType(type(value)):
                return value() # Method
            else:
                try: return value() # Callable object?
                except AttributeError, ev:
                    try:
                        tb=sys.exc_traceback
                        if hasattr(sys, 'exc_info'): tb=sys.exc_info()[2]
                        if hasattr(value,'__call__'):
                            raise AttributeError, ev, tb
                    finally: tb=None
                except TypeError: pass # Obviously not.

        return value
    else:
        return ''

#
# Returns an RDF description of a properties of an object for use in ZopeStudio
#
def zsPropertiesRDF(self, REQUEST):
    """ Return an RDF description of a properties of an object for use in
        ZopeStudio """

    self = REQUEST.PARENTS[0]

    REQUEST.RESPONSE.setHeader('content-type', 'text/xml')
    
    propertiesURI = REQUEST.URL1 + ':properties'
    
    propertiesList = map(lambda propId, 
        baseURI=propertiesURI: """<rdf:li resource="%s/%s" />""" % (baseURI,
            propId), self.propertyIds())
    propertiesList = join(propertiesList, '\n')

    propertiesRDF = map(lambda propertyId, S=self, 
        baseURI=propertiesURI: propertyRDFSnippet(S,propertyId, baseURI),
            self.propertyIds())
    propertiesRDF = join(propertiesRDF,'\n')
    

    output = """<?xml version="1.0"?>
<rdf:RDF xml:lang="en"
         xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
         xmlns:dc="http://purl.org/metadata/dublin_core#"
         xmlns:zope="http://www.zope.org/Resources/Mozilla/RDF#"
         xmlns:nc="http://home.netscape.com/NC-rdf#">
         
 <rdf:Seq about="%s">
    %s
 </rdf:Seq>
    %s
</rdf:RDF>""" % (propertiesURI, propertiesList, propertiesRDF)

    return output;

# Shove the above method in the Application class
OFS.Application.Application.zsPropertiesRDF = zsPropertiesRDF

#
# Returns an RDF description of one property
#
def propertyRDFSnippet(object,property,uri):
    propertyInfo = object.propdict()[property]
    propertyId = propertyInfo['id']
    propertyType = propertyInfo['type']
    propertyValue = object.getProperty(property)

    if propertyInfo.has_key('mode'):
        propertyReadOnly = "true"
        propertyAllowDelete = "false"
        
        propertyMode = propertyInfo['mode']
        if find(propertyMode,"w"): propertyReadOnly="false"
        if find(propertyMode,"d"): propertyAllowDelete="true"
    else:
        propertyReadOnly = "false"
        propertyAllowDelete = "true"
        
    if propertyType == 'text':
        propertyValue = string.replace(propertyValue,'\n',',')
            
    commonAttributes = """about="%s/%s" dc:Identifier="%s" zope:Type="property"
        zope:PropertyId="%s" zope:PropertyType="%s" zope:PropertyValue="%s"
        zope:PropertyReadOnly="%s" 
        zope:PropertyAllowDelete="%s" """ % (uri, property, property,
            propertyId, propertyType, propertyValue, propertyReadOnly,
            propertyAllowDelete)
                                    
    if propertyType == 'selection' or propertyType == 'multiple selection':
        listElements = ""
        listElementDescriptions = ""
        propertyUri = uri + '/' + property

        for possibleValue in getvalue(object, propertyInfo['select_variable']):
                
            propertySelected='false'
            
            if propertyType=='selection':
                if possibleValue == propertyValue: propertySelected = 'true'
            elif propertyType=='multiple selection':
                if possibleValue in propertyValue: propertySelected = 'true'
                    
            propertyUri = uri + '/' + property
            listElements = listElements + """<rdf:li resource="%s/%s"/> """ % (
                propertyUri, possibleValue)
            listElementDescriptions = listElementDescriptions + """
                <rdf:Description about="%s/%s" zope:SelectOption="%s" 
                zope:Selected="%s"
                zope:Type="selectoption"/>""" % (propertyUri,
                    possibleValue, possibleValue, propertySelected)
                                    
            propertyRDF = """\n<rdf:Seq %s zope:PropertySelectVariable="%s">
                %s
                </rdf:Seq>
                %s""" % (commonAttributes, propertyInfo['select_variable'],
                    listElements, listElementDescriptions)

    elif propertyType=='lines' or propertyType=='tokens':
        listElements = map(lambda possibleValue: 
            """<rdf:li zope:ItemValue="%s"/>""" % (possibleValue),
                propertyValue)
        listElements = join(listElements, '\n')

        propertyRDF = """\n<rdf:Seq %s>
            %s
            </rdf:Seq>""" % (commonAttributes, listElements)
        
    else:
        propertyRDF = """\n<rdf:Description %s/>""" % (commonAttributes)

    return propertyRDF
    
